import json

from django import forms
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from wagtail import VERSION as WAGTAIL_VERSION
from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.widgets import BaseChooser, BaseChooserAdapter


if WAGTAIL_VERSION >= (7, 1):
    from wagtail.admin.telepath import register
else:
    from wagtail.telepath import register

from wagtailmedia.models import get_media_model


class AdminMediaChooser(BaseChooser):
    media_type = None
    choose_one_text = _("Choose a media item")
    choose_another_text = _("Choose another media item")
    link_to_chosen_text = _("Edit this media item")
    icon = "media"
    classname = "media-chooser"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.model = get_media_model()

    @property
    def chooser_modal_url_name(self):
        if self.media_type:
            return "wagtailmedia:chooser_typed"
        return "wagtailmedia:chooser"

    def get_chooser_modal_url(self):
        if self.media_type:
            return reverse("wagtailmedia:chooser_typed", args=(self.media_type,))
        return reverse("wagtailmedia:chooser")

    def render_js_init(self, id_, name, value):
        return f"createMediaChooser({json.dumps(id_)});"

    @property
    def media(self):
        tab_js = [] if WAGTAIL_VERSION >= (7, 1) else ["wagtailmedia/js/tabs.js"]
        js = [
            *tab_js,
            "wagtailmedia/js/media-chooser-modal.js",
            "wagtailmedia/js/media-chooser.js",
        ]
        return forms.Media(js=js)


class AdminAudioChooser(AdminMediaChooser):
    media_type = "audio"
    icon = "wagtailmedia-audio"
    choose_one_text = _("Choose audio")
    choose_another_text = _("Choose another audio item")
    link_to_chosen_text = _("Edit this audio item")


class AdminVideoChooser(AdminMediaChooser):
    media_type = "video"
    icon = "wagtailmedia-video"
    choose_one_text = _("Choose video")
    choose_another_text = _("Choose another video")
    link_to_chosen_text = _("Edit this video")


class MediaChooserAdapter(BaseChooserAdapter):
    js_constructor = "wagtailmedia.MediaChooser"

    @cached_property
    def media(self):
        return forms.Media(
            js=[
                versioned_static("wagtailmedia/js/media-chooser-telepath.js"),
            ]
        )


register(MediaChooserAdapter(), AdminMediaChooser)
register(MediaChooserAdapter(), AdminAudioChooser)
register(MediaChooserAdapter(), AdminVideoChooser)



"""
The wagtailmedia settings are namespaced in the WAGTAILMEDIA setting.
For example your project's `settings.py` file might look like this:
WAGTAILMEDIA = {
    "MEDIA_MODEL": "mymedia.CustomMedia",
    # ...
}
This module provides the `wagtailmedia_settings` object, that is used to access
the settings. It checks for user settings first, with fallback to defaults.
"""

import warnings

from django.conf import settings
from django.test.signals import setting_changed


DEFAULTS = {
    "MEDIA_MODEL": "wagtailmedia.Media",
    "MEDIA_FORM_BASE": "",
    "AUDIO_EXTENSIONS": ["aac", "aiff", "flac", "m4a", "m4b", "mp3", "ogg", "wav"],
    "VIDEO_EXTENSIONS": [
        "avi",
        "h264",
        "m4v",
        "mkv",
        "mov",
        "mp4",
        "mpeg",
        "mpg",
        "ogv",
        "webm",
    ],
}

# List of settings that have been deprecated
DEPRECATED_SETTINGS = []

# List of settings that have been removed
# note: use a tuple of (setting, deprecation warning from deprecation.py)
REMOVED_SETTINGS = []


class WagtailMediaSettings:
    """
    A settings object that allows the wagtailmedia settings to be accessed as
    properties. For example:
        from wagtailmedia.settings import wagtailmedia_settings
        print(wagtailmedia_settings.MEDIA_MODEL)
    Note:
    This is an internal class that is only compatible with settings namespaced
    under the WAGTAILMEDIA name. It is not intended to be used by 3rd-party
    apps, and test helpers like `override_settings` may not work as expected.
    """

    def __init__(self, user_settings=None, defaults=None):
        if user_settings:
            self._user_settings = self.__check_user_settings(user_settings)
        self.defaults = defaults or DEFAULTS
        self._cached_attrs = set()

    @property
    def user_settings(self):
        if not hasattr(self, "_user_settings"):
            self._user_settings = self.__check_user_settings(
                getattr(settings, "WAGTAILMEDIA", {})
            )
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError(f"Invalid wagtailmedia setting: '{attr}'")

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Cache the result
        self._cached_attrs.add(attr)
        setattr(self, attr, val)
        return val

    def __check_user_settings(self, user_settings):
        for setting, category in DEPRECATED_SETTINGS:
            if setting in user_settings or hasattr(settings, setting):
                new_setting = setting.replace("WAGTAILMEDIA_", "")
                warnings.warn(
                    f"The '{setting}' setting is deprecated and will be removed in the next release, "
                    f'use WAGTAILMEDIA["{new_setting}"] instead.',
                    category=category,
                    stacklevel=2,
                )
                user_settings[new_setting] = user_settings[setting]
        for setting in REMOVED_SETTINGS:
            if setting in user_settings:
                raise RuntimeError(
                    f"The '{setting}' setting has been removed. "
                    f"Please refer to the wagtailmedia documentation for available settings."
                )
        return user_settings

    def reload(self):
        for attr in self._cached_attrs:
            delattr(self, attr)
        self._cached_attrs.clear()
        if hasattr(self, "_user_settings"):
            delattr(self, "_user_settings")


wagtailmedia_settings = WagtailMediaSettings(None, DEFAULTS)


def reload_wagtailmedia_settings(*args, **kwargs):
    setting = kwargs["setting"]
    if setting == "WAGTAILMEDIA":
        wagtailmedia_settings.reload()


setting_changed.connect(reload_wagtailmedia_settings)



# Generated by Django 2.2.8 on 2020-08-17 15:30

import django.core.validators

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailmedia", "0003_copy_media_permissions_to_collections"),
    ]

    operations = [
        migrations.AlterField(
            model_name="media",
            name="duration",
            field=models.FloatField(
                blank=True,
                default=0,
                help_text="Duration in seconds",
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="duration",
            ),
        ),
    ]



from django.apps import AppConfig
from django.db.models import ForeignKey


class WagtailMediaAppConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "wagtailmedia"
    label = "wagtailmedia"
    verbose_name = "Wagtail media"

    def ready(self):
        from wagtail.admin.compare import register_comparison_class

        from .edit_handlers import MediaFieldComparison
        from .models import get_media_model
        from .signal_handlers import register_signal_handlers

        register_signal_handlers()

        # Set up image ForeignKeys to use ImageFieldComparison as the comparison class
        # when comparing page revisions
        register_comparison_class(
            ForeignKey, to=get_media_model(), comparison_class=MediaFieldComparison
        )



from django.template import Library
from wagtail.utils.version import get_main_version


register = Library()


@register.simple_tag
def wagtail_version_gte(version: str) -> bool:
    return get_main_version() >= version



# Generated by Django 5.2 on 2025-05-06 16:58

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailmedia", "0004_duration_optional_floatfield"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="media",
            options={"verbose_name": "media", "verbose_name_plural": "media items"},
        ),
    ]



from django.db import transaction
from django.db.models.signals import post_delete

from wagtailmedia.models import get_media_model


def delete_files(instance):
    # Pass false so FileField doesn't save the model.
    instance.file.delete(False)
    if instance.thumbnail:
        instance.thumbnail.delete(False)


def post_delete_file_cleanup(instance, **kwargs):
    transaction.on_commit(lambda: delete_files(instance))


def register_signal_handlers():
    Media = get_media_model()
    post_delete.connect(post_delete_file_cleanup, sender=Media)



from django.core.paginator import Paginator
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils.text import capfirst
from django.utils.translation import gettext as _
from django.views.decorators.vary import vary_on_headers
from wagtail.admin import messages
from wagtail.admin.admin_url_finder import AdminURLFinder
from wagtail.admin.auth import PermissionPolicyChecker, permission_denied
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.models import popular_tags_for_model
from wagtail.models import Collection, Page
from wagtail.search.backends import get_search_backends

from wagtailmedia.forms import get_media_form
from wagtailmedia.models import get_media_model
from wagtailmedia.permissions import permission_policy
from wagtailmedia.utils import paginate


permission_checker = PermissionPolicyChecker(permission_policy)


@permission_checker.require_any("add", "change", "delete")
@vary_on_headers("X-Requested-With")
def index(request):
    Media = get_media_model()

    # Get media files (filtered by user permission)
    media = permission_policy.instances_user_has_any_permission_for(
        request.user, ["change", "delete"]
    )

    # Ordering
    if request.GET.get("ordering") in ["title", "-title", "-created_at", "created_at"]:
        ordering = request.GET["ordering"]
    else:
        ordering = "-created_at"
    media = media.order_by(ordering)

    # Filter by collection
    current_collection = None
    collection_id = request.GET.get("collection_id")
    if collection_id:
        try:
            current_collection = Collection.objects.get(id=collection_id)
            media = media.filter(collection=current_collection)
        except (ValueError, Collection.DoesNotExist):
            pass

    # Search
    query_string = ""
    if "q" in request.GET and request.GET.get("q").strip():
        form = SearchForm(request.GET, placeholder=_("Search media files"))
        if form.is_valid():
            query_string = form.cleaned_data["q"]
            media = media.search(query_string)
    else:
        form = SearchForm(placeholder=_("Search media"))

    # Filter by tag
    if current_tag := request.GET.get("tag"):
        try:
            media = media.filter(tags__name=current_tag)
        except AttributeError:
            current_tag = None

    # Pagination
    paginator, media = paginate(request, media)

    collections = permission_policy.collections_user_has_any_permission_for(
        request.user, ["add", "change"]
    )
    if len(collections) < 2:
        collections = None

    # Create response
    if request.headers.get("x-requested-with") == "XMLHttpRequest":
        return render(
            request,
            "wagtailmedia/media/results.html",
            {
                "ordering": ordering,
                "media_files": media,
                "query_string": query_string,
                "is_searching": bool(query_string),
                "collections": collections,
            },
        )
    else:
        return render(
            request,
            "wagtailmedia/media/index.html",
            {
                "ordering": ordering,
                "media_files": media,
                "query_string": query_string,
                "is_searching": bool(query_string),
                "search_form": form,
                "popular_tags": popular_tags_for_model(Media),
                "current_tag": current_tag,
                "user_can_add": permission_policy.user_has_permission(
                    request.user, "add"
                ),
                "collections": collections,
                "current_collection": current_collection,
            },
        )


@permission_checker.require("add")
def add(request, media_type):
    Media = get_media_model()
    MediaForm = get_media_form(Media)

    if request.method == "POST":
        media = Media(uploaded_by_user=request.user, type=media_type)
        form = MediaForm(request.POST, request.FILES, instance=media, user=request.user)
        if form.is_valid():
            form.save()

            # Reindex the media entry to make sure all tags are indexed
            for backend in get_search_backends():
                backend.add(media)

            messages.success(
                request,
                _("Media file '{0}' added.").format(media.title),
                buttons=[
                    messages.button(
                        reverse("wagtailmedia:edit", args=(media.id,)), _("Edit")
                    )
                ],
            )
            return redirect("wagtailmedia:index")
        else:
            messages.error(
                request, _("The media file could not be saved due to errors.")
            )
    else:
        media = Media(uploaded_by_user=request.user, type=media_type)
        form = MediaForm(user=request.user, instance=media)

    return render(
        request,
        "wagtailmedia/media/add.html",
        {
            "form": form,
            "media_type": media_type,
        },
    )


@permission_checker.require("change")
def edit(request, media_id):
    Media = get_media_model()
    MediaForm = get_media_form(Media)

    media = get_object_or_404(Media, id=media_id)

    if not permission_policy.user_has_permission_for_instance(
        request.user, "change", media
    ):
        return permission_denied(request)

    if request.method == "POST":
        original_file = media.file
        form = MediaForm(request.POST, request.FILES, instance=media, user=request.user)
        if form.is_valid():
            if "file" in form.changed_data:
                # if providing a new media file, delete the old one.
                # NB Doing this via original_file.delete() clears the file field,
                # which definitely isn't what we want...
                original_file.storage.delete(original_file.name)
            media = form.save()

            # Reindex the media entry to make sure all tags are indexed
            for backend in get_search_backends():
                backend.add(media)

            messages.success(
                request,
                _("Media file '{0}' updated").format(media.title),
                buttons=[
                    messages.button(
                        reverse("wagtailmedia:edit", args=(media.id,)), _("Edit")
                    )
                ],
            )
            return redirect("wagtailmedia:index")
        else:
            messages.error(request, _("The media could not be saved due to errors."))
    else:
        form = MediaForm(instance=media, user=request.user)

    filesize = None

    # Get file size when there is a file associated with the Media object
    if media.file:
        try:
            filesize = media.file.size
        except OSError:
            # File doesn't exist
            pass

    if not filesize:
        messages.error(
            request,
            _(
                "The file could not be found. Please change the source or delete the media file"
            ),
            buttons=[
                messages.button(
                    reverse("wagtailmedia:delete", args=(media.id,)), _("Delete")
                )
            ],
        )

    return render(
        request,
        "wagtailmedia/media/edit.html",
        {
            "media": media,
            "filesize": filesize,
            "form": form,
            "user_can_delete": permission_policy.user_has_permission_for_instance(
                request.user, "delete", media
            ),
        },
    )


@permission_checker.require("delete")
def delete(request, media_id):
    Media = get_media_model()
    media = get_object_or_404(Media, id=media_id)

    if not permission_policy.user_has_permission_for_instance(
        request.user, "delete", media
    ):
        return permission_denied(request)

    if request.method == "POST":
        media.delete()
        messages.success(request, _("Media file '{0}' deleted.").format(media.title))
        return redirect("wagtailmedia:index")

    return render(
        request,
        "wagtailmedia/media/confirm_delete.html",
        {
            "media": media,
        },
    )


@permission_checker.require_any("add", "change", "delete")
def usage(request, media_id):
    Media = get_media_model()
    media_item = get_object_or_404(Media, id=media_id)

    paginator = Paginator(media_item.get_usage(), per_page=20)
    object_page = paginator.get_page(request.GET.get("p"))

    # Add edit URLs to each source object
    url_finder = AdminURLFinder(request.user)
    results = []
    for entity, references in object_page:
        edit_url = url_finder.get_edit_url(entity)
        if edit_url is None:
            label = _("(Private %s)") % entity._meta.verbose_name
            edit_link_title = None
        else:
            label = str(entity)
            edit_link_title = _("Edit this %s") % entity._meta.verbose_name

        if isinstance(entity, Page):
            verbose_name = capfirst(entity.specific_class._meta.verbose_name)
        else:
            verbose_name = capfirst(entity._meta.verbose_name)

        results.append((label, edit_url, edit_link_title, verbose_name, references))

    return render(
        request,
        "wagtailmedia/media/usage.html",
        {"media_item": media_item, "results": results, "object_page": object_page},
    )



from wagtail.api.v2.filters import FieldsFilter, OrderingFilter, SearchFilter
from wagtail.api.v2.views import BaseAPIViewSet

from ..models import get_media_model
from .serializers import MediaItemSerializer


class MediaAPIViewSet(BaseAPIViewSet):
    base_serializer_class = MediaItemSerializer
    filter_backends = [FieldsFilter, OrderingFilter, SearchFilter]
    body_fields = BaseAPIViewSet.body_fields + [
        "title",
        "width",
        "height",
        "media_type",
        "collection",
    ]
    meta_fields = BaseAPIViewSet.meta_fields + [
        "tags",
        "download_url",
    ]
    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        "media_type",
        "title",
        "width",
        "height",
        "tags",
        "collection",
        "thumbnail",
        "download_url",
    ]
    nested_default_fields = BaseAPIViewSet.nested_default_fields + [
        "title",
        "collection",
        "download_url",
    ]
    name = "media"
    model = get_media_model()



from django.contrib import admin

from wagtailmedia.settings import wagtailmedia_settings


if wagtailmedia_settings.MEDIA_MODEL == "wagtailmedia.Media":
    # Only expose the package-provided media class in the Django admin if the installation
    # does not provide its own custom media class in order to avoid confusion.
    from wagtailmedia.models import Media

    admin.site.register(Media)



from typing import TYPE_CHECKING

from django.forms import ModelChoiceField
from django.template.loader import render_to_string
from django.utils.functional import cached_property
from wagtail.admin.compare import BlockComparison
from wagtail.blocks import ChooserBlock

from .utils import format_audio_html, format_video_html


if TYPE_CHECKING:
    from .widgets import AdminAudioChooser, AdminVideoChooser


class AbstractMediaChooserBlock(ChooserBlock):
    def __init__(
        self, required=True, help_text=None, validators=(), media_type=None, **kwargs
    ):
        super().__init__(
            required=required, help_text=help_text, validators=validators, **kwargs
        )
        self.media_type = media_type

    @cached_property
    def target_model(self):
        from wagtailmedia.models import get_media_model

        return get_media_model()

    @cached_property
    def field(self):
        if not self.media_type:
            return super().field

        return ModelChoiceField(
            queryset=self.target_model.objects.filter(type=self.media_type),
            widget=self.widget,
            required=self._required,
            validators=self._validators,
            help_text=self._help_text,
        )

    @cached_property
    def widget(self):
        from wagtailmedia.widgets import AdminMediaChooser

        return AdminMediaChooser()

    def render_basic(self, value, context=None):
        raise NotImplementedError(
            f"You need to implement {self.__class__.__name__}.render_basic"
        )

    def get_comparison_class(self) -> type["MediaChooserBlockComparison"]:
        return MediaChooserBlockComparison


class MediaChooserBlockComparison(BlockComparison):
    def htmlvalue(self, value) -> str:
        return render_to_string(
            "wagtailmedia/widgets/compare.html",
            {
                "media_item_a": self.block.render_basic(value),
                "media_item_b": self.block.render_basic(value),
            },
        )

    def htmldiff(self) -> str:
        return render_to_string(
            "wagtailmedia/widgets/compare.html",
            {
                "media_item_a": self.block.render_basic(self.val_a),
                "media_item_b": self.block.render_basic(self.val_b),
            },
        )


class AudioChooserBlock(AbstractMediaChooserBlock):
    def __init__(self, required=True, help_text=None, validators=(), **kwargs):
        super().__init__(
            required=required,
            help_text=help_text,
            validators=validators,
            media_type="audio",
            **kwargs,
        )

    @cached_property
    def widget(self) -> "AdminAudioChooser":
        from wagtailmedia.widgets import AdminAudioChooser

        return AdminAudioChooser()

    def render_basic(self, value, context=None) -> str:
        if not value:
            return ""

        if value.type != self.media_type:
            return ""

        return format_audio_html(value)

    class Meta:
        icon = "wagtailmedia-audio"


class VideoChooserBlock(AbstractMediaChooserBlock):
    def __init__(self, required=True, help_text=None, validators=(), **kwargs):
        super().__init__(
            required=required,
            help_text=help_text,
            validators=validators,
            media_type="video",
            **kwargs,
        )

    @cached_property
    def widget(self) -> "AdminVideoChooser":
        from wagtailmedia.widgets import AdminVideoChooser

        return AdminVideoChooser()

    def render_basic(self, value, context=None) -> str:
        if not value:
            return ""

        if value.type != self.media_type:
            return ""

        return format_video_html(value)

    class Meta:
        icon = "wagtailmedia-video"
